from cpl_core.configuration import ConfigurationABC
from cpl_core.environment import ApplicationEnvironmentABC
from cpl_core.mailing import EMailClientABC, EMailClientSettings
from cpl_translation import TranslatePipe
from flask import request, jsonify, Response

from bot_api.abc.auth_service_abc import AuthServiceABC
from bot_api.api import Api
from bot_api.filter.auth_user_select_criteria import AuthUserSelectCriteria
from bot_api.json_processor import JSONProcessor
from bot_api.logging.api_logger import ApiLogger
from bot_api.model.auth_user_dto import AuthUserDTO
from bot_api.model.reset_password_dto import ResetPasswordDTO
from bot_api.model.token_dto import TokenDTO
from bot_api.model.update_auth_user_dto import UpdateAuthUserDTO
from bot_api.route.route import Route
from bot_data.model.auth_role_enum import AuthRoleEnum


class AuthController:
    BasePath = "/api/auth"

    def __init__(
        self,
        config: ConfigurationABC,
        env: ApplicationEnvironmentABC,
        logger: ApiLogger,
        t: TranslatePipe,
        api: Api,
        mail_settings: EMailClientSettings,
        mailer: EMailClientABC,
        auth_service: AuthServiceABC,
    ):
        self._config = config
        self._env = env
        self._logger = logger
        self._t = t
        self._api = api
        self._mail_settings = mail_settings
        self._mailer = mailer
        self._auth_service = auth_service

    @Route.get(f"{BasePath}/users")
    @Route.authorize(role=AuthRoleEnum.admin)
    async def get_all_users(self) -> Response:
        result = await self._auth_service.get_all_auth_users_async()
        return jsonify(result.select(lambda x: x.to_dict()).to_list())

    @Route.post(f"{BasePath}/users/get/filtered")
    @Route.authorize(role=AuthRoleEnum.admin)
    async def get_filtered_users(self) -> Response:
        dto: AuthUserSelectCriteria = JSONProcessor.process(
            AuthUserSelectCriteria, request.get_json(force=True, silent=True)
        )
        result = await self._auth_service.get_filtered_auth_users_async(dto)
        result.result = result.result.select(lambda x: x.to_dict()).to_list()
        return jsonify(result.to_dict())

    @Route.get(f"{BasePath}/users/get/<email>")
    @Route.authorize
    async def get_user_from_email(self, email: str) -> Response:
        result = await self._auth_service.get_auth_user_by_email_async(email)
        return jsonify(result.to_dict())

    @Route.get(f"{BasePath}/users/find/<email>")
    @Route.authorize
    async def find_user_from_email(self, email: str) -> Response:
        result = await self._auth_service.find_auth_user_by_email_async(email)
        return jsonify(result.to_dict())

    @Route.post(f"{BasePath}/register")
    async def register(self):
        dto: AuthUserDTO = JSONProcessor.process(AuthUserDTO, request.get_json(force=True, silent=True))
        self._auth_service.add_auth_user(dto)
        return "", 200

    @Route.post(f"{BasePath}/register-by-id/<id>")
    async def register_id(self, id: str):
        result = await self._auth_service.confirm_email_async(id)
        return jsonify(result)

    @Route.post(f"{BasePath}/login")
    async def login(self) -> Response:
        dto: AuthUserDTO = JSONProcessor.process(AuthUserDTO, request.get_json(force=True, silent=True))
        result = await self._auth_service.login_async(dto)
        return jsonify(result.to_dict())

    @Route.get(f"{BasePath}/verify-login")
    async def verify_login(self):
        token = None
        result = False
        if "Authorization" in request.headers:
            bearer = request.headers.get("Authorization")
            token = bearer.split()[1]

        if token is not None:
            result = self._auth_service.verify_login(token)

        return jsonify(result)

    @Route.post(f"{BasePath}/forgot-password/<email>")
    async def forgot_password(self, email: str):
        await self._auth_service.forgot_password_async(email)
        return "", 200

    @Route.post(f"{BasePath}/confirm-forgot-password/<id>")
    async def confirm_forgot_password(self, id: str):
        result = await self._auth_service.confirm_forgot_password_async(id)
        return jsonify(result.to_dict())

    @Route.post(f"{BasePath}/reset-password")
    async def reset_password(self):
        dto: ResetPasswordDTO = JSONProcessor.process(ResetPasswordDTO, request.get_json(force=True, silent=True))
        await self._auth_service.reset_password_async(dto)
        return "", 200

    @Route.post(f"{BasePath}/update-user")
    @Route.authorize
    async def update_user(self):
        dto: UpdateAuthUserDTO = JSONProcessor.process(UpdateAuthUserDTO, request.get_json(force=True, silent=True))
        await self._auth_service.update_user_async(dto)
        return "", 200

    @Route.post(f"{BasePath}/update-user-as-admin")
    @Route.authorize(role=AuthRoleEnum.admin)
    async def update_user_as_admin(self):
        dto: UpdateAuthUserDTO = JSONProcessor.process(UpdateAuthUserDTO, request.get_json(force=True, silent=True))
        await self._auth_service.update_user_as_admin_async(dto)
        return "", 200

    @Route.post(f"{BasePath}/refresh")
    @Route.authorize
    async def refresh(self) -> Response:
        dto: TokenDTO = JSONProcessor.process(TokenDTO, request.get_json(force=True, silent=True))
        result = await self._auth_service.refresh_async(dto)
        return jsonify(result.to_dict())

    @Route.post(f"{BasePath}/revoke")
    async def revoke(self):
        dto: TokenDTO = JSONProcessor.process(TokenDTO, request.get_json(force=True, silent=True))
        await self._auth_service.revoke_async(dto)
        return "", 200

    @Route.post(f"{BasePath}/delete-user")
    @Route.authorize(role=AuthRoleEnum.admin)
    async def delete_user(self):
        dto: AuthUserDTO = JSONProcessor.process(AuthUserDTO, request.get_json(force=True, silent=True))
        await self._auth_service.delete_auth_user_async(dto)
        return "", 200

    @Route.post(f"{BasePath}/delete-user-by-mail/<email>")
    @Route.authorize(role=AuthRoleEnum.admin)
    async def delete_user_by_mail(self, email: str):
        await self._auth_service.delete_auth_user_by_email_async(email)
        return "", 200
